@@ -1,10 +1,13 @@
+# -*- coding: utf-8 -*-
 """pyprof2html - Profile data convert to HTML.
 
 This script is converted to HTML file from Python's cProfile and
 hotshot profiling data.
 """
 
+import codecs
 import os
+import re
 import sys
 import time
 from hotshot import log, stats
@@ -13,7 +16,7 @@ from pstats import Stats
 from jinja2 import Template
 
 
-__version__ = '0.1.6'
+__version__ = '0.1.7'
 __licence__ = 'New BSD License'
 __author__ = 'Hideo Hattori <syobosyobo@gmail.com>'
 
@@ -187,7 +190,9 @@ XHTML_BODY_TEMPLATE = u"""\
                     {% if item['linelink'][0] != "<" %}
                         ><a href="{{ item['linelink']|escape }}"
                         >{{ item['func'][0]|escape }}</a
-                        > : {{ item['func'][1] }} &#40;
+                        > : <a href="{{ item['linelink']|escape }}#line{{ \
+                                item['func'][1] }}"
+                        >{{ item['func'][1] }}</a> &#40;
                          {{ item['func'][2]|escape }}&nbsp;&#41;</span></td>
                     {% else %}
                         >{{ item['func'][0]|escape }} :
@@ -239,10 +244,10 @@ XHTML_LINEBODY_TEMPLATE = u"""\
     <th class="prof" align="center"><span class="tdmg">calls</span></th>
     <th class="prof" align="center"><span class="tdmg">code</span></th>
     </tr>
-    {% for item in profdata %}
+    {% for item in profdata['data'] %}
         <tr>
-        <td class="profline_r"><span class="tdms" style="color:#999999;"
-        >{{ loop.index }}</span></td>
+        <td id="line{{ loop.index }}" class="profline_r"
+        ><span class="tdms" style="color:#999999;">{{ loop.index }}</span></td>
         {% if item['sec'] == 'None' %}
             <td class="profline_r"><span class="tdms"></span></td>
             <td class="profline_r"><span class="tdms"></span></td>
@@ -304,7 +309,21 @@ def mapping_table(target, nall):
     return levelmap[len(levelmap)-1]
 
 
-class Converter:
+def detect_filecodec(lines):
+    """detect to Python Source Code Encodings.
+
+    Only strict check. see PEP 0263...
+    """
+    re_codec = re.compile("coding[:=]\s*([-\w.]+)")
+    for cnt, line in enumerate(lines):
+        if re_codec.search(line):
+            return re_codec.findall(line)[0]
+        if cnt >= 2:
+            break
+    return None
+
+
+class Converter(object):
     """output to HTML from profile data
     """
 
@@ -320,7 +339,9 @@ class Converter:
             if check_hotlinetimings(dump[102:108]):
                 self.profline = log.LogReader(filename)
                 self.proftype = 'line'
-                self.tmplline = Template(XHTML_LINEBODY_TEMPLATE)
+                self.tmplline = Template(XHTML_HEADER_TEMPLATE + \
+                                         XHTML_LINEBODY_TEMPLATE + \
+                                         XHTML_FOOTER_TEMPLATE)
         else:
             self.prof = Stats(filename)
             self.tmpl = Template(XHTML_BODY_TEMPLATE)
@@ -330,51 +351,49 @@ class Converter:
         self.outputtype = 'html'
         self.functions_number = 20
 
-    def _printhtml_source(self, filename, profs):
+    def _printhtml_source(self, filename, profs, outputfile):
         """printing one file profile line. return value is html render
         strings.
         """
-        num = 0
         result = []
-        try:
-            lines = open(filename).readlines()
-        except IOError:
+        if not os.path.exists(filename):
             return ""
-        codec = None
-        for line in lines:
-            ## TODO: Oops... I don't absolute to this 'UnicodeDecodeError'.
-            ## darty hack
-            try:
-                line.decode(sys.getdefaultencoding())
-            except UnicodeDecodeError:
-                line = "Unicode Decoding Error!!!"
-            num += 1
-            if num in profs:
-                result.append({'sec':profs[num]['sec'],
-                               'cnt':profs[num]['cnt'], 'line':line})
+        filecodec = detect_filecodec(open(filename).readlines())
+        if filecodec is None:
+            fileobj = open(filename)
+        else:
+            fileobj = codecs.open(filename, 'r', filecodec)
+        for num, line in enumerate(fileobj):
+            if num+1 in profs:
+                result.append({'sec': profs[num+1]['sec'],
+                               'cnt': profs[num+1]['cnt'],
+                               'line': line})
             else:
-                result.append({'sec':'None', 'line':line})
+                result.append({'sec': 'None', 'line': line})
         titletext = "pyprof2html - %s" % filename
-        return self.tmplline.render(title=titletext,
-                                    proftime=self.proftime,
-                                    reporttime=self.reporttime,
-                                    filename=filename,
-                                    profdata=result,
-                                    codec=codec)
+        renderdict = {'title': titletext,
+                      'proftime': self.proftime,
+                      'reporttime': self.reporttime,
+                      'filename': filename,
+                      'profdata': {'data': result,
+                                   'totaltime': "%8.4lf" % self.prof.total_tt,
+                                   'totalcalls': self.prof.total_calls}}
+        if filecodec is None:
+            print >> outputfile, self.tmplline.render(renderdict)
+        else:
+            self.tmplline.stream(renderdict).dump(outputfile, 'utf-8')
 
     def _print_source(self, filename, profs):
         """printing one file profile line. return value is text strings.
         """
         num = 0
         result = []
-        try:
-            lines = open(filename).readlines()
-        except IOError:
+        if not os.path.exists(filename):
             return ""
         result.append("="*60 + "\n")
         result.append(filename + "\n")
         result.append("="*60 + "\n")
-        for line in lines:
+        for line in open(filename):
             num += 1
             if num in profs:
                 result.append(" %3.4lfs | %7dn | %s" % (profs[num]['sec'],
@@ -391,7 +410,6 @@ class Converter:
         info = {}
         filecnt = 0
         for i, prof in enumerate(profs):
-            print prof
             filename = prof[0]
             if (not sources.count(filename)) or (len(profs) == (i+1)):
                 if info != {}:
@@ -399,17 +417,15 @@ class Converter:
                         out_filename = "%s/%s.html" % (self.output_dir,
                                        sources[filecnt].replace('/', '_'))
                         outputfile = open(out_filename, 'w')
-                        print >> outputfile, XHTML_HEADER_TEMPLATE
-                        print >> outputfile, self._printhtml_source(
-                                                  sources[filecnt], info)
-                        print >> outputfile, XHTML_FOOTER_TEMPLATE
+                        self._printhtml_source(sources[filecnt],
+                                               info, outputfile)
                         outputfile.close()
                     else:
                         print self._print_source(sources[filecnt], info)
                     filecnt += 1
                 info = {}
                 sources.append(filename)
-            info[prof[1]] = {'sec':prof[2], 'cnt':prof[3]}
+            info[prof[1]] = {'sec': prof[2], 'cnt': prof[3]}
 
     def _analyze_profline(self):
         """analyzed to hotshot linetimings data.
@@ -569,4 +585,3 @@ def main():
 
 if __name__ == '__main__':
     sys.exit(main())
-
